/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.cropcircle;

import java.util.List;
import javax.persistence.Query;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.SensorDataFile;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.sensor.SensorService;

/**
 * Service for doing database transactions on cropcircle data
 * @author Merijn Zengers
 */
public class CropcircleServiceImpl extends SensorService {

    private static final AppLogger LOG = AppLogFactory.getLogger(CropcircleServiceImpl.class);
    private static final String LOGKEY_PROCESSING_ERRORMSG = "cropcircle.service.logerror.bulk_persist_error";

    @Override
    protected String getModuleName() {
        return CropcircleModule.MODULE_NAME;
    }

    /**
     * Method to bulk persist.
     * Clears the entitymanager after persisting. Only to be used in single thread.
     * WARNING all non persisted entity's will be cleared from the entitymanager !!!
     * @Return list<CropcircleSensorData>: returns the failing CropcircleSensorData's.
     * @param sensorDataList: List of CropcircleSensorData to bulkPersist
     * @param blockSize: the size of the blocks between persists
     */
    public void bulkPersist(List<CropcircleSensorData> sensorDataList, int blockSize) {
        int merged = 0;
        try{
            startTransaction();        
            for (CropcircleSensorData sensorData : sensorDataList) {
                merge(sensorData);
                if(merged + 1 % blockSize == 0){
                    commit();
                }
                merged++;
            }
            if(getEntityManager().getTransaction().isActive()){
                commit();
            }
            //TODO: try to find another solotion for memory management
            getEntityManager().clear();
        } catch(Exception ex){
            getEntityManager().clear();
            if(blockSize == 1){
                LOG.error(LOGKEY_PROCESSING_ERRORMSG, sensorDataList.get(merged));
                sensorDataList.remove(merged);
                bulkPersist(sensorDataList,1);
            } else {
                bulkPersist(sensorDataList,1);
            }
        }   
    }

    public void delete(CropcircleSensorData toBeRemoved) {
        getEntityManager().remove(toBeRemoved);
    }
    
    protected void removeSensorDataFile(SensorDataFile toBeRemoved) {
        getEntityManager().remove(toBeRemoved);
    }
        

    public CropcircleSensorData findSensorDataById(long id) {
        startTransaction();
        return getEntityManager().find(CropcircleSensorData.class, id);
    }

    public List<CropcircleSensorData> findAllCropcircleSensorData() {
        startTransaction();
        Query query = this.getEntityManager().createQuery("select c from CropcircleSensorData c");
        return query.getResultList();
    }

    public void updateSensorMetadata(SensorDataFile dataFile, SensorMetadata newSensorMetadata) {
        startTransaction();
        if (dataFile.getId() == 0) {
            throw new IllegalArgumentException("Cannot update a SensorDataFile if it does not exist");
        } else {
            if (newSensorMetadata.getId() != 0) {
                dataFile.setMetadata(newSensorMetadata);
            } else {
                SensorMetadata existing = findMetaDataByMetadata(newSensorMetadata.getDateFormat(), newSensorMetadata.getSensorHeight(), newSensorMetadata.getNumberOfSensors(), newSensorMetadata.getSensorSpread(), newSensorMetadata.getSensorType(), newSensorMetadata.getSpectralChannels());
                if (existing != null) {
                    dataFile.setMetadata(existing);
                    dataFile = getEntityManager().merge(dataFile);
                } else {
                    getEntityManager().persist(newSensorMetadata);
                    dataFile.setMetadata(newSensorMetadata);
                }
            }
            commit();
        }
    }
}
